@@ -4,12 +4,13 @@
 import numpy as np
 from numpy import interp
 import warnings
+import collections
 
 from sklearn.base import BaseEstimator, MultiOutputMixin
 from sklearn.utils.validation import (check_X_y, check_array, check_is_fitted,
                                       _check_sample_weight)
-
 from sklearn.linear_model._base import _preprocess_data, _rescale_data
+from sklearn.model_selection import GridSearchCV
 
 # Module-wide constants
 BIG_BIAS = 10e3
@@ -17,64 +18,17 @@ SMALL_BIAS = 10e-3
 BIAS_STEP = 0.2
 
 
-__all__ = ["fracridge", "vec_len", "FracRidge"]
+__all__ = ["fracridge", "vec_len", "FracRidgeRegressor",
+           "FracRidgeRegressorCV"]
 
 
-def fracridge(X, y, fracs=None, tol=1e-6, jit=True):
+def _do_svd(X, y, jit=True):
     """
-    Approximates alpha parameters to match desired fractions of OLS length.
-
-    Parameters
-    ----------
-    X : ndarray, shape (n, p)
-        Design matrix for regression, with n number of
-        observations and p number of model parameters.
-
-    y : ndarray, shape (n, b)
-        Data, with n number of observations and b number of targets.
-
-    fracs : float or 1d array, optional
-        The desired fractions of the parameter vector length, relative to
-        OLS solution. If 1d array, the shape is (f,).
-        Default: np.arange(.1, 1.1, .1)
-
-    jit : bool, optional
-        Whether to speed up computations by using a just-in-time compiled
-        version of core computations. This may not work well with very large
-        datasets. Default: True
-
-    Returns
-    -------
-    coef : ndarray, shape (p, f, b)
-        The full estimated parameters across units of measurement for every
-        desired fraction.
-    alphas : ndarray, shape (f, b)
-        The alpha coefficients associated with each solution
-    Examples
-    --------
-    Generate random data:
-    >>> np.random.seed(0)
-    >>> y = np.random.randn(100)
-    >>> X = np.random.randn(100, 10)
-
-    Calculate coefficients with naive OLS:
-    >>> coef = np.linalg.inv(X.T @ X) @ X.T @ y
-    >>> print(np.linalg.norm(coef))  # doctest: +NUMBER
-    0.35
-
-    Call fracridge function:
-    >>> coef2, alpha = fracridge(X, y, 0.3)
-    >>> print(np.linalg.norm(coef2))  # doctest: +NUMBER
-    0.10
-    >>> print(np.linalg.norm(coef2) / np.linalg.norm(coef))  # doctest: +NUMBER
-    0.3
-
-    Calculate coefficients with naive RR:
-    >>> alphaI = alpha * np.eye(X.shape[1])
-    >>> coef3 = np.linalg.inv(X.T @ X + alphaI) @ X.T @ y
-    >>> print(np.linalg.norm(coef2 - coef3))  # doctest: +NUMBER
-    0.0
+    Helper function to produce SVD outputs
     """
+    if len(y.shape) == 1:
+        y = y[:, np.newaxis]
+
     # Per default, we'll try to use the jit-compiled SVD, which should be
     # more performant:
     use_scipy = False
@@ -94,10 +48,95 @@ def fracridge(X, y, fracs=None, tol=1e-6, jit=True):
         from scipy.linalg import svd  # noqa
         svd = partial(svd, full_matrices=False)
 
+    if X.shape[0] > X.shape[1]:
+        uu, ss, v_t = svd(X.T @ X)
+        selt = np.sqrt(ss)
+        if y.shape[-1] >= X.shape[0]:
+            ynew = (1/selt) @ v_t @ X.T @ y
+        else:
+            ynew = np.diag(1./selt) @ v_t @ (X.T @ y)
+
+    else:
+        # This rotates the targets by the unitary matrix uu.T:
+        uu, selt, v_t = svd(X)
+        ynew = uu.T @ y
+
+    ols_coef = (ynew.T / selt).T
+
+    return selt, v_t, ols_coef
+
+
+def fracridge(X, y, fracs=None, tol=1e-10, jit=True):
+    """
+    Approximates alpha parameters to match desired fractions of OLS length.
+
+    Parameters
+    ----------
+    X : ndarray, shape (n, p)
+        Design matrix for regression, with n number of
+        observations and p number of model parameters.
+
+    y : ndarray, shape (n, b)
+        Data, with n number of observations and b number of targets.
+
+    fracs : float or 1d array, optional
+        The desired fractions of the parameter vector length, relative to
+        OLS solution. If 1d array, the shape is (f,). This input is required
+        to be sorted. Otherwise, raises ValueError.
+        Default: np.arange(.1, 1.1, .1).
+
+    jit : bool, optional
+        Whether to speed up computations by using a just-in-time compiled
+        version of core computations. This may not work well with very large
+        datasets. Default: True
+
+    Returns
+    -------
+    coef : ndarray, shape (p, f, b)
+        The full estimated parameters across units of measurement for every
+        desired fraction.
+    alphas : ndarray, shape (f, b)
+        The alpha coefficients associated with each solution
+
+    Examples
+    --------
+
+    Generate random data:
+
+    >>> np.random.seed(0)
+    >>> y = np.random.randn(100)
+    >>> X = np.random.randn(100, 10)
+
+    Calculate coefficients with naive OLS:
+
+    >>> coef = np.linalg.inv(X.T @ X) @ X.T @ y
+    >>> print(np.linalg.norm(coef))  # doctest: +NUMBER
+    0.35
+
+    Call fracridge function:
+
+    >>> coef2, alpha = fracridge(X, y, 0.3)
+    >>> print(np.linalg.norm(coef2))  # doctest: +NUMBER
+    0.10
+    >>> print(np.linalg.norm(coef2) / np.linalg.norm(coef))  # doctest: +NUMBER
+    0.3
+
+    Calculate coefficients with naive RR:
+
+    >>> alphaI = alpha * np.eye(X.shape[1])
+    >>> coef3 = np.linalg.inv(X.T @ X + alphaI) @ X.T @ y
+    >>> print(np.linalg.norm(coef2 - coef3))  # doctest: +NUMBER
+    0.0
+    """
     if fracs is None:
         fracs = np.arange(.1, 1.1, .1)
 
-    if not hasattr(fracs, "__len__"):
+    if hasattr(fracs, "__len__"):
+        if np.any(np.diff(fracs) < 0):
+            raise ValueError("The `frac` inputs to the `fracridge` function ",
+                             f"must be sorted. You provided: {fracs}")
+
+    else:
         fracs = [fracs]
     fracs = np.array(fracs)
 
@@ -108,15 +147,8 @@ def fracridge(X, y, fracs=None, tol=1e-6, jit=True):
     bb = y.shape[-1]
     ff = fracs.shape[0]
 
-    uu, selt, v_t = svd(X)
-
-    # This rotates the targets by the unitary matrix uu.T:
-    ynew = uu.T @ y
-    del uu
-
-    # Solve OLS for the rotated problem and replace y:
-    ols_coef = (ynew.T / selt).T
-    del ynew
+    # Calculate the rotation of the data
+    selt, v_t, ols_coef = _do_svd(X, y, jit=jit)
 
     # Set solutions for small eigenvalues to 0 for all targets:
     isbad = selt < tol
@@ -178,48 +210,74 @@ def fracridge(X, y, fracs=None, tol=1e-6, jit=True):
     return coef.squeeze(), alphas
 
 
-class FracRidge(BaseEstimator, MultiOutputMixin):
+class FracRidgeRegressor(BaseEstimator, MultiOutputMixin):
     """
-    Fractional Ridge Regression estimator
-
-
-
     Parameters
     ----------
     fracs : float or sequence
         The desired fractions of the parameter vector length, relative to
-        OLS solution. If 1d array, the shape is (f,).
+        OLS solution.
         Default: np.arange(.1, 1.1, .1)
 
+    fit_intercept : bool, optional
+        Whether to fit an intercept term. Default: False.
+
+    normalize : bool, optional
+        Whether to normalize the columns of X. Default: False.
+
+    copy_X : bool, optional
+        Whether to make a copy of the X matrix before fitting. Default: True.
+
+    tol : float, optional.
+        Tolerance under which singular values of the X matrix are considered
+        to be zero. Default: 1e-10.
+
+    jit : bool, optional.
+        Whether to use jit-accelerated implementation. Default: True.
+
+    Attributes
+    ----------
+    coef_ : ndarray, shape (p, f, b)
+        The full estimated parameters across units of measurement for every
+        desired fraction. Where p number of model parameters, f number of
+        fractions and b number of targets.
+
+
+    alpha_ : ndarray, shape (f, b)
+        The alpha coefficients associated with each solution. Where f number
+        of fractions and b number of targets.
 
     Examples
     --------
+
     Generate random data:
+
     >>> np.random.seed(1)
     >>> y = np.random.randn(100)
     >>> X = np.random.randn(100, 10)
 
     Calculate coefficients with naive OLS:
+
     >>> coef = np.linalg.inv(X.T @ X) @ X.T @ y
 
     Initialize the estimator with a single fraction:
-    >>> fr = FracRidge(fracs=0.3)
+
+    >>> fr = FracRidgeRegressor(fracs=0.3)
 
     Fit estimator:
+
     >>> fr.fit(X, y)
-    FracRidge(fracs=0.3)
+    FracRidgeRegressor(fracs=0.3)
 
     Check results:
+
     >>> coef_ = fr.coef_
     >>> alpha_ = fr.alpha_
     >>> print(np.linalg.norm(coef_) / np.linalg.norm(coef)) # doctest: +NUMBER
     0.29
     """
-    def _more_tags(self):
-        return {'multioutput': True}
-
     def __init__(self, fracs=None, fit_intercept=False, normalize=False,
-                 copy_X=True, tol=1e-6, jit=True):
+                 copy_X=True, tol=1e-10, jit=True):
         self.fracs = fracs
         self.fit_intercept = fit_intercept
         self.normalize = normalize
@@ -227,7 +285,10 @@ class FracRidge(BaseEstimator, MultiOutputMixin):
         self.tol = tol
         self.jit = jit
 
-    def fit(self, X, y, sample_weight=None):
+    def _validate_input(self, X, y, sample_weight=None):
+        """
+        Helper function to validate the inputs
+        """
         X, y = check_X_y(X, y, y_numeric=True, multi_output=True)
 
         if sample_weight is not None:
@@ -242,19 +303,27 @@ class FracRidge(BaseEstimator, MultiOutputMixin):
         if sample_weight is not None:
             # Sample weight can be implemented via a simple rescaling.
             X, y = _rescale_data(X, y, sample_weight)
+        return X, y, X_offset, y_offset, X_scale
 
-        self.is_fitted_ = True
+    def fit(self, X, y, sample_weight=None):
+        X, y, X_offset, y_offset, X_scale = self._validate_input(
+            X, y, sample_weight=sample_weight)
         coef, alpha = fracridge(X, y, fracs=self.fracs, tol=self.tol,
                                 jit=self.jit)
         self.alpha_ = alpha
         self.coef_ = coef
         self._set_intercept(X_offset, y_offset, X_scale)
+        self.is_fitted_ = True
         return self
 
     def predict(self, X):
         X = check_array(X, accept_sparse=True)
         check_is_fitted(self, 'is_fitted_')
-        pred = np.tensordot(X, self.coef_, axes=(1))
+        if len(self.coef_.shape) == 0:
+            pred_coef = self.coef_[np.newaxis]
+        else:
+            pred_coef = self.coef_
+        pred = np.tensordot(X, pred_coef, axes=(1))
         if self.fit_intercept:
             pred = pred + self.intercept_
         return pred
@@ -272,60 +341,121 @@ class FracRidge(BaseEstimator, MultiOutputMixin):
         else:
             self.intercept_ = 0.
 
+    def score(self, X, y, sample_weight=None):
+        """
+        Score the fracridge fit
+        """
+        from sklearn.metrics import r2_score
+        y_pred = self.predict(X)
+        if len(y_pred.shape) > len(y.shape):
+            y = y[..., np.newaxis]
+        y = np.broadcast_to(y, y_pred.shape)
+        return r2_score(y, y_pred, sample_weight=sample_weight)
+
+    def _more_tags(self):
+        return {'multioutput': True}
+
+
+class FracRidgeRegressorCV(FracRidgeRegressor):
+    """
+    Uses :class:`sklearn.model_selection.GridSearchCV` to find the best
+    value of `frac` given the data, using cross-validation.
+
+    Parameters
+    ----------
+    frac_grid : sequence or float, optional
+        The values of frac to consider. Default: np.arange(.1, 1.1, .1)
+
+    fit_intercept : bool, optional
+        Whether to fit an intercept term. Default: False.
+
+    normalize : bool, optional
+        Whether to normalize the columns of X. Default: False.
+
+    copy_X : bool, optional
+        Whether to make a copy of the X matrix before fitting. Default: True.
+
+    tol : float, optional.
+        Tolerance under which singular values of the X matrix are considered
+        to be zero. Default: 1e-10.
+
+    jit : bool, optional.
+        Whether to use jit-accelerated implementation. Default: True.
+
+    cv : int, cross-validation generator or an iterable
+        See https://scikit-learn.org/stable/modules/generated/sklearn.model_selection.GridSearchCV.html  # noqa
+
+    scoring : str, callable, list/tuple or dict, default=None
+        See https://scikit-learn.org/stable/modules/generated/sklearn.model_selection.GridSearchCV.html  # noqa
+
+    Attributes
+    ----------
+    best_frac_ : float
+        The best fraction as determined by cross-validation.s
+
+    alpha_ : ndarray, shape (b)
+        The alpha coefficients associated with this fraction for each
+        target.
+
+    coef_ : ndarray, shape (p, b)
+        The coefficients corresponding to the best solution. Where p number
+        of parameters and b number of targets.
+
+    Examples
+    --------
+    Generate random data:
+
+    >>> np.random.seed(1)
+    >>> y = np.random.randn(100)
+    >>> X = np.random.randn(100, 10)
+
+    Fit model with cross-validation:
+
+    >>> frcv = FracRidgeRegressorCV()
+    >>> frcv.fit(X, y)
+    FracRidgeRegressorCV(frac_grid=array([0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1. ]))
+    >>> print(frcv.best_frac_)
+    0.1
+    """
+    def __init__(self, frac_grid=None, fit_intercept=False, normalize=False,
+                 copy_X=True, tol=1e-10, jit=True, cv=None, scoring=None):
+
+        self.frac_grid = frac_grid
+        if self.frac_grid is None:
+            self.frac_grid = np.arange(.1, 1.1, .1)
+        super().__init__(self, fit_intercept=False, normalize=False,
+                         copy_X=True, tol=tol, jit=True)
+        self.cv = cv
+        self.scoring = scoring
+
+    def fit(self, X, y, sample_weight=None):
+        X, y, _, _, _ = self._validate_input(
+            X, y, sample_weight=sample_weight)
+
+        parameters = {'fracs': self.frac_grid}
+        gs = GridSearchCV(
+                FracRidgeRegressor(
+                    fit_intercept=self.fit_intercept,
+                    normalize=self.normalize,
+                    copy_X=self.copy_X,
+                    tol=self.tol,
+                    jit=self.jit),
+                parameters, cv=self.cv, scoring=self.scoring)
+
+        gs.fit(X, y, sample_weight=sample_weight)
+        estimator = gs.best_estimator_
+        self.best_score_ = gs.best_score_
+        self.coef_ = estimator.coef_
+        self.intercept_ = estimator.intercept_
+        self.best_frac_ = estimator.fracs
+        self.alpha_ = estimator.alpha_
+        self.is_fitted_ = True
+
+        return self
+
+    def _more_tags(self):
+        return {'multioutput': True}
+
 
 def vec_len(vec, axis=0):
-    return np.sqrt((vec * vec).sum(axis=axis))
-
-
-def optimize_for_frac(X, fracs):
-    """
-    Empirically find the alpha that gives frac reduction in vector length of
-    the solution
-
-    """
-    u, s, v = svd(X)
-
-    val1 = 10e3 * s[0] ** 2  # Huge bias
-    val2 = 10e-3 * s[-1] ** 2  # Tiny bias
-
-    alphas = np.concatenate(
-        [np.array([0]), 10 ** np.arange(np.floor(np.log10(val2)),
-                                        np.ceil(np.log10(val1)), 0.1)])
-
-    results = np.zeros(alphas.shape[0])
-    for ii, alpha in enumerate(alphas):
-        results[ii] = frac_reduction(X, alpha, s=s)
-
-    return interp1d(results, alphas, bounds_error=False, fill_value="extrapolate")(np.asarray(fracs))
-
-
-def frac_reduction(X, alpha, s=None):
-    """
-    Calculates the expected fraction reduction in the length of the
-    coefficient vector $\beta$ from OLS to ridge, given a design matrix X and
-    a regularization metaparameter alpha.
-    """
-    if s is None:
-        u, s, v = svd(X)
-    new = s / (s ** 2 + alpha)
-    olslen = np.sqrt(np.sum((1 / s) ** 2))
-    rrlen = np.sqrt(np.sum(new ** 2))
-    return rrlen / olslen
-
-
-def frac_reduction_flat(X, alpha, s=None):
-    """
-    This is the version that assumes a flat eigenvalue spectrum
-    """
-    if s is None:
-        u, s, v = svd(X)
-    return np.mean(s ** 2 / (s ** 2 + alpha))
-
-
-def reg_alpha_flat(X, gamma, s=None):
-    """
-    This is the version that assumes a flat eigenvalue spectrum
-    """
-    if s is None:
-        u, s, v = svd(X)
-    return (s ** 2) * (1 / gamma - 1)
+    return np.sqrt((vec * vec).sum(axis=axis))
\ No newline at end of file
